<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 12, 2020
 */

/**
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Backups_Api extends Model_Api
{

    const API_DATA_STATE = 'state';
    const API_DATA_CONTEXT = 'context';
    const API_DATA_CONTEXT_ID = 'context_id';
    const API_DATA_POLICY_ID = 'policy_id';
    const API_DATA_ID = 'id';

    const DEFAULT_NAME = 'InterWorx Default';

    private $_cookieClient;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_cookieClient = $this->_getCookieClient();
    }

    /**
     * Get list of backups.
     *
     * @param string $instanceId
     * @return array
     */
    public function getBackups($instanceId)
    {
        // should be fine to use internal api here - instance id is what we expect.
        // If we do update this endpoint, we need to make sure it does not break recovery - it relies on this format.
        $endpoint = "/bc/api/vault_manager/v1/backups?count=1&embed=archive&includeActions=1&limit=25&order=desc(created)&resourceId=$instanceId";
        $backups = $this->_cookieClient->sendAuthenticatedGetRequest($endpoint);
        return $backups;
    }

    /**
     * Create draft for backup plan.
     *
     * @deprecated
     * @param string $resourceId
     * @return array
     */
    public function createDraft($resourceId)
    {
        $config = $this->_getCreateDraftConfig($resourceId);
        $apiEndpoint = '/bc/api/ams/backup/plan_drafts';

        $res = $this->_cookieClient->sendAuthenticatedPostRequest($apiEndpoint, $config);
        return $res;
    }

    /**
     * Get config object for create draft endpoint.
     *
     * @param string $resourceId
     * @return array
     */
    private function _getCreateDraftConfig($resourceId)
    {
        // TODO enable daily backup = defaults to mon-fri
        $defaultStartTime = array(
            'hour' => 2,
            'minute' => 0,
            'second' => 0
        );

        $data = array(
            'language' => 'en',
            'sourceType' => 'machines',
            'bootableMediaPlan' => false,
            'defaultStartTime' => $defaultStartTime,
            'resourcesIds' => array($resourceId)
        );

        return array('data' => $data);
    }

    /**
     * Change to daily schedule.
     *
     * @param string $draftId
     */
    public function updateDraftDailyBackups($draftId)
    {

        $data = $this->_getDataDailyBackups();
        $config = array('data' => $data);
        $url = "/bc/api/ams/backup/plan_drafts/$draftId/scheme";

        $this->_cookieClient->sendAuthenticatedPutRequest($url, $config);
    }

    /**
     * @return array
     */
    private function _getDataDailyBackups()
    {
        $defaultStartTime = $this->_getDraftDefaultStartTime();
        $daysOfWeek = $this->_getDraftDaysOfWeek();

        $schedule = array(
            'type' => 'daily',
            'startAt' => $defaultStartTime,
            'daysOfWeek' => $daysOfWeek,
            'runLater' => false,
            'machineWake' => false,
            'preventFromSleeping' => true
        );

        return array(
            'type' => 'alwaysIncremental',
            'schedule' => $schedule
        );
    }

    /**
     * @return array
     */
    private function _getDraftDefaultStartTime()
    {
        return array(
            'hour' => 2,
            'minute' => 0,
            'second' => 0
        );
    }

    /**
     * @return array
     */
    private function _getDraftDaysOfWeek()
    {
        return array('sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday');
    }


    /**
     * Create backup plan.
     *
     * @param string $draftId
     * @return array
     */
    public function createPlan($draftId)
    {
        $this->_renameDraft($draftId);
        $this->_editPrePostCommands($draftId);

        $plan = $this->_createPlan($draftId);
        return $plan;
    }

    /**
     * Rename backup plan. Does not seem to be another way to name the plan.
     *
     * @param string $draftId
     */
    private function _renameDraft($draftId)
    {
        $params = "/bc/api/ams/backup/plan_drafts/$draftId/parameters";

        $data = array('planName' => self::DEFAULT_NAME);
        $config = array('data' => $data);

        $this->_cookieClient->sendAuthenticatedPutRequest($params, $config);
    }


    /**
     * Edit plan draft to include pre-post commands & save plan.
     *
     * @param string $draftId
     */
    private function _editPrePostCommands($draftId)
    {
        $apiEndpoint = "/bc/api/ams/backup/plan_drafts/$draftId/options";
        $config = array(
            'data' => $this->_getPrePostCommandsData()
        );

        $this->_cookieClient->sendAuthenticatedPutRequest($apiEndpoint, $config);
    }

    /**
     * Get draft data for request.
     *
     * @return array
     */
    private function _getPrePostCommandsData()
    {
        $preCommands = $this->_getCommandsConfig('webcpprecapture');
        $postCommands = $this->_getCommandsConfig('webcppostcapture');

        return array(
            'prePostCommands' => array(
                'useDefaultCommands' => false,
                'postCommands' => $postCommands,
                'preCommands' => $preCommands,
                'usePostCommands' => true,
                'usePreCommands' => true
            )
        );
    }

    /**
     * Get pre/post command config.
     *
     * @param string $prePostCapture
     * @return array
     */
    private function _getCommandsConfig($prePostCapture)
    {
        return array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '{RESOURCE_ID}',
            'waitCommandComplete' => true,
            'command' => "/usr/lib/Acronis/BackupAndRecovery/$prePostCapture"
        );
    }

    /**
     * create plan
     *
     * @param string $draftId
     * @return array
     */
    private function _createPlan($draftId)
    {
        $config = $this->_getDraftIdConfig($draftId);
        $apiEndpoint = '/bc/api/ams/backup/plan_drafts_operations/create_plan';
        $res = $this->_cookieClient->sendAuthenticatedPostRequest($apiEndpoint, $config);
        return $res;
    }

    /**
     * Get draft id config.
     *
     * @param string $draftId
     * @return array
     */
    private function _getDraftIdConfig($draftId)
    {
        $data = array(
            'draftId' => $draftId
        );

        return array(
            'data' => $data
        );
    }

    /**
     * get plan draft.
     *
     * @param string $draftId
     */
    public function getPlanDraft($draftId)
    {
        $url = "/bc/api/ams/backup/plan_drafts/$draftId";
        return $this->_cookieClient->sendAuthenticatedGetRequest($url);
    }
}
